package com.anagog.anagog_flutter

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.annotation.NonNull
import com.anagog.anagog_flutter.flutter_bridge.EventEmitter
import com.anagog.anagog_flutter.model.Micromoment
import com.anagog.anagog_flutter.plugin_callbacks.PluginCampaignTriggeredHandler
import com.anagog.anagog_flutter.plugin_callbacks.PluginNotificationApprovalHandler
import com.anagog.anagog_flutter.plugin_callbacks.PluginNotificationClickedHandler
import com.anagog.anagog_flutter.plugin_callbacks.PluginReportSnapshotHandler
import com.anagog.jedai.anagog_api.AnagogApi
import com.anagog.jedai.common.ApplicationEvent
import com.anagog.jedai.common.poi.Point
import com.anagog.jedai.core.logger.JedAILogger
import com.anagog.jedai.core.sdk_state.SdkState
import com.google.gson.Gson

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler

/** AnagogFlutterPlugin */
class AnagogFlutterPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    private val sLogger: JedAILogger by lazy { JedAILogger.getLogger("com.anagog.anagog_flutter.AnagogFlutterPlugin") }

    init {
        sLogger.info("init: ")
    }

    private lateinit var channel: MethodChannel
    private var eventsChannel: EventChannel? = null
    private var context: Context? = null
    private var activity: Activity? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        sLogger.info("onAttachedToEngine ")
        setMethodChannel(flutterPluginBinding)
        setEventChannel(flutterPluginBinding)
        AnagogApi.obtainEventStream(EventEmitter)
        context = flutterPluginBinding.applicationContext
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        sLogger.info("onDetachedFromEngine ")
        channel.setMethodCallHandler(null)
        context = null
        eventsChannel = null
        AnagogApi.removeEventListener(EventEmitter)
        EventEmitter.onDetachFromEngine()
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull resultHandler: MethodChannel.Result) {
        sLogger.info("onMethodCall: method: ${call.method}, arguments: ${call.arguments}")
        when (call.method) {
            "simulateCampaign" -> {
                val campaignIdentifier = call.argument<String>("campaign_identifier")
                AnagogApi.simulateCampaign(campaignIdentifier, onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "syncCampaigns" -> {
                AnagogApi.syncCampaigns(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "setUserDefinedString" -> {
                val arguments = call.arguments
                if (arguments is Map<*, *>) {
                    val key = arguments["key"].toString()
                    val value = arguments["value"].toString()
                    AnagogApi.setUserDefinedString(key, value, onError = {
                        resultHandler.error("", it.message, null)
                    }) {
                        resultHandler.success(null)
                    }
                }
            }
            "getUserDefinedString" -> {
                val key = call.arguments as String?
                key?.let {
                    AnagogApi.getUserDefinedString(key, onError = {
                        resultHandler.error("", it.message, null)
                    }) { res ->
                        Log.i("AnagogFlutterPlugin", "getUserDefinedString: $key: result -> $res")
                        resultHandler.success(res)
                    }
                } ?: kotlin.run {
                    resultHandler.error("", "invalid parameters", null)
                }
            }
            "setUserDefinedInteger" -> {
                val arguments = call.arguments
                if (arguments is Map<*, *>) {
                    val key = arguments["key"].toString()
                    val value = arguments["value"].toString()
                    AnagogApi.setUserDefinedInteger(key, value.toInt(), onError = {
                        resultHandler.error("", it.message, null)
                    }) {
                        resultHandler.success(null)
                    }
                }
            }
            "getUserDefinedInteger" -> {
                val key = call.arguments as String?
                key?.let {
                    AnagogApi.getUserDefinedInteger(it, onError = { e ->
                        resultHandler.error("", e.message, null)
                    }) { res ->
                        resultHandler.success(res)
                    }
                } ?: kotlin.run {
                    resultHandler.error("", "invalid parameters", null)
                }
            }
            "setUserDefinedDecimal" -> {
                val arguments = call.arguments
                if (arguments is Map<*, *>) {
                    val key = arguments["key"].toString()
                    val value = arguments["value"].toString()
                    AnagogApi.setUserDefinedDecimal(key, value.toDouble(), onError = {
                        resultHandler.error("", it.message, null)
                    }) {
                        resultHandler.success(null)
                    }
                }
            }
            "getUserDefinedDecimal" -> {
                val key = call.arguments as String?
                key?.let {
                    AnagogApi.getUserDefinedDecimal(it, onError = { e ->
                        resultHandler.error("", e.message, null)
                    }) { res ->
                        resultHandler.success(res)
                    }
                } ?: kotlin.run {
                    resultHandler.error("","invalid parameter", null)
                }
            }
            "fireMicromoment" -> {
                val micromoment = Gson().fromJson(call.arguments.toString(), Micromoment::class.java)

                val identifier = micromoment.identifier
                val appEvent = ApplicationEvent(identifier)

                micromoment.textParameters.forEach {
                    appEvent.addParameter(it.key, it.value)
                }

                micromoment.numericParameters.forEach { entry ->
                    appEvent.addParameter(entry.key, (entry.value).toFloat())
                }

                micromoment.booleanParameters.forEach { entry ->
                    appEvent.addParameter(entry.key, entry.value)
                }

                micromoment.locationParameters.forEach { entry ->
                    val point = Point(entry.value.latitude, entry.value.longitude)
                    appEvent.addParameter(entry.key, point)
                }

                AnagogApi.fireMicromoment(appEvent, onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "share" -> {
                try {
                    context?.let { c ->
                        AnagogApi.share(c, onError = { e ->
                            resultHandler.error("", e.message, null)
                        }){
                            resultHandler.success(null)
                        }
                    }
                } catch (e: Throwable) {
                    resultHandler.error("", e.message, null)
                }
            }
            "getVersion" -> {
                AnagogApi.getVersion(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(it)
                }
            }
            "requestMicrosegmentsSnapshotReport" -> {
                AnagogApi.requestMicrosegmentsSnapshotReport(onError = {
                    resultHandler.error("", it.message, null)
                }, onSuccess = {
                    resultHandler.success(null)
                })
            }
            "forceScheduleReports" -> {
                AnagogApi.forceScheduleReports(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "setUserInteractionEvent" -> {
                val arguments = call.arguments
                if (arguments is Map<*, *>) {
                    val campaignIdentifier = arguments["campaign_identifier"].toString()
                    val activityType = arguments["activity_type"].toString()
                    if (campaignIdentifier.isNotEmpty() && activityType.isNotEmpty()) {
                        AnagogApi.sendUserInteractionEvent(campaignIdentifier, activityType, onError = {
                            resultHandler.error("", it.message, null)
                        }, onSuccess = {
                            resultHandler.success(null)
                        })
                    } else {
                        resultHandler.error("Invalid arguments for userInteractionEvent", "Make sure you provide map like {'campaign_identifier': '..', 'activity_type':'..'}", null)
                    }
                } else {
                    resultHandler.error("Invalid arguments for userInteractionEvent", "Make sure you provide map like {'campaign_identifier': '..', 'activity_type':'..'}", null)
                }
            }
            "getCampaigns" -> {
                AnagogApi.getCampaigns(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(it)
                }
            }
            "getJedAIStatus" -> {
                resultHandler.success(AnagogApi.getJedAIStatus() == SdkState.RUNNING)
            }
            "enableJedAI" -> {
                AnagogApi.startAnagogSdk()
                resultHandler.success(null)
            }
            "disableJedAI" -> {
                AnagogApi.stopAnagogSdk(false)
            }
            "disableAndPurgeJedAI" -> {
                AnagogApi.stopAnagogSdk(true)
            }
            "openDebuggingScreen" -> {
                try {
                    this.activity?.let {
                        AnagogApi.openDebuggingScreen(it, onError = { e ->
                            resultHandler.error("", "${e.message}", null)
                        }){
                            resultHandler.success(null)
                        }
                    }
                } catch (e: Throwable) {
                    resultHandler.error("", "${e.message}, make sure you've included anagog debugging module in app build.gradle", null)
                }
            }
            "onboardingStart" -> {
                sLogger.info("onboardingStart: ")
                AnagogApi.startOnboarding(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "onboardingComplete" -> {
                AnagogApi.onboardingCompleted(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "onboardingMarkAsCompleted" -> {
                AnagogApi.markAsCompletedOnboarding(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "onboardingReset" -> {
                AnagogApi.resetOnboarding(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "enterPage" -> {
                val pageName = (call.arguments as Map<*, *>?)?.get("pageName") as String?
                pageName?.let {
                    AnagogApi.enterPage(pageName = it, onError = {e ->
                        resultHandler.error("", e.message, null)
                    }) {
                        resultHandler.success(null)
                    }
                } ?: kotlin.run {
                    resultHandler.error("", "invalid parameters", null)
                }
            }
            "exitPage" -> {
                AnagogApi.exitPage(onError = {
                    resultHandler.error("", it.message, null)
                }) {
                    resultHandler.success(null)
                }
            }
            "logInfo" -> {
                val arguments = call.arguments
                if (arguments is Map<*, *>) {
                    val message = arguments["message"].toString()
                    if (message.isNotEmpty()) {
                        sLogger.info(message)
                        resultHandler.success(null)
                    } else {
                        resultHandler.error("Invalid arguments for logInfo", "Make sure you provide an appropriate message", null)
                    }
                } else {
                    resultHandler.error("Invalid arguments for logInfo", "Make sure you provide an appropriate message", null)
                }

            }
            "logError" -> {
                val arguments = call.arguments
                if (arguments is Map<*, *>) {
                    val message = arguments["message"].toString()
                    if (message.isNotEmpty()) {
                        sLogger.error(message)
                        resultHandler.success(null)
                    } else {
                        resultHandler.error("Invalid arguments for logError", "Make sure you provide an appropriate message", null)
                    }
                } else {
                    resultHandler.error("Invalid arguments for logError", "Make sure you provide an appropriate message", null)
                }
            }
            else -> resultHandler.notImplemented()
        }
    }

    private fun setMethodChannel(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, METHOD_CHANNEL)
        channel.setMethodCallHandler(this)
        //sdkWrapper.engineAttached()
        sLogger.info("setMethodChannel: attached")
    }

    private fun setEventChannel(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        eventsChannel = EventChannel(flutterPluginBinding.binaryMessenger, EVENT_CHANNEL)
        eventsChannel?.setStreamHandler(EventEmitter)
    }

    companion object {
        internal const val METHOD_CHANNEL = "com.anagog.flutter/method_channel"
        internal const val EVENT_CHANNEL = "com.anagog.flutter/method_channel/events"

        @JvmStatic
        fun setOnNotificationClickHandler(handler: PluginNotificationClickedHandler) {
            AnagogApi.setNotificationClickHandler { executed, action, payload ->
                handler.onNotificationClicked(executed, action, payload)
            }
        }

        @JvmStatic
        fun setNotificationApprovalHandler(handler: PluginNotificationApprovalHandler) {
            AnagogApi.setNotificationApprovalHandler {
                handler.onNotificationApproval(it)
            }
        }

        @JvmStatic
        fun setCampaignTriggeredHandler(handler: PluginCampaignTriggeredHandler) {
            AnagogApi.setCampaignTriggeredHandler { action, campaign ->
                handler.onCampaignTriggered(action, campaign)
            }
        }

        @JvmStatic
        fun setOnSnapshotReportHandler(handler: PluginReportSnapshotHandler) {
            AnagogApi.setReportSnapshotHandler {
                handler.onReportSnapshot(it)
            }
        }
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        sLogger.info("onAttachedToActivity")

        this.activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
        this.activity = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        sLogger.info("onReattachedToActivityForConfigChanges")
        this.activity = binding.activity
    }

    override fun onDetachedFromActivity() {
        sLogger.info("onDetachedFromActivity")
        this.activity = null
    }
}
