package com.anagog.anagog_flutter.flutter_bridge

import android.os.Handler
import android.os.Looper
import com.anagog.jedai.anagog_api.callbacks.AnagogSdkMessageListener
import com.anagog.jedai.anagog_api.model.Message
import com.anagog.jedai.core.logger.JedAILogger
import com.anagog.jedai.core.sdk_state.SdkState
import io.flutter.plugin.common.EventChannel

internal object EventEmitter : EventChannel.StreamHandler, AnagogSdkMessageListener {
    private val sLogger: JedAILogger by lazy { JedAILogger.getLogger("com.anagog.anagog_flutter.EventEmitter") }

    private var eventSink: EventChannel.EventSink? = null
    private val pendingEventsQueue = mutableListOf<Map<String, Any?>>()

    private val handler = Handler(Looper.getMainLooper())

    private fun emitEvent(event: Map<String, Any?>) {
        sLogger.error { "About to emit event: $event" }
        eventSink?.let {
            handler.post { //Posting events on Main Thread (Flutter mandatory)
                try {
                    it.success(event) //Send event to Flutter code!
                } catch (e: Throwable) {
                    e.printStackTrace()
                    sLogger.error { "Something went wrong when trying to emit event: $event from native side to Flutter. Please contact Anagog support. ${e.message}" }
                }
            }
        } ?: run {
            sLogger.error { "Sink not ready, add event to pendingEventsQueue.." }
            pendingEventsQueue.add(event)
        }
    }

    fun onDetachFromEngine() {
        eventSink?.endOfStream()
        eventSink = null
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {  //This one will be called by Flutter when Dart code listen to events
        sLogger.info { "EventSink onListen.. arguments $arguments, pendingEventsQueue has ${pendingEventsQueue.size} events, about to flush pendingEventsQueue" }
        eventSink = events
        pendingEventsQueue.forEach { //There are events that wait for Flutter, flush
            sLogger.error { "flush pendingEventsQueue: $it" }
            eventSink?.success(it) //Deliver undelivered campaign
        }
        pendingEventsQueue.clear()
    }

    override fun onCancel(arguments: Any?) {
        eventSink = null
    }

    override fun onNewMessage(message: Message) { //From Facade
        sLogger.info { "onNewMessage: $message" }
        when(message){
            is Message.OnSdkStatusChanged -> emitEvent(mapOf(Pair("action", "onJedAIStatusChange"), Pair("payload", mapOf(Pair("status", message.status == SdkState.RUNNING)))))
            is Message.OnCampaignTriggered -> emitEvent(mapOf(Pair("action", "OnCampaignTriggered"), Pair("payload", message.userObject)))
            is Message.OnSnapshotReport -> emitEvent(mapOf(Pair("action", "OnSnapshotReport"), Pair("payload", message.snapshotReport)))
            is Message.OnNotificationClicked -> emitEvent(mapOf(Pair("action", "onNotificationClick"), Pair("payload", message.payload)))
        }
    }
}