//
//  AnagogPackageOptionsHandler.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Foundation
import JedAIAppKit
import JedAIKit

open class AnagogPackageOptionsHandler: JedAIAppOptionsHandler {
    
    internal var url: URL? = Bundle.main.url(forResource: "anagog_config", withExtension: "json")
    private var configs: [String: Any] = [:]
    
    internal var deprecatedIntegrationConfigs: [String: Any] = [:]
    
    public override init() {
        super.init()
    }

    private func loadFromFile() {
        guard let url = url else {
            print("[package] Info: Configuration File URL is nil")
            setDeprecatedIntegrationConfigs()
            return
        }
        guard FileManager.default.fileExists(atPath: url.path) else {
            print("[package] Info: Configuration File doesn't exist \(url.path)")
            setDeprecatedIntegrationConfigs()
            return
        }

        guard
            let data = try? Data(contentsOf: url),
            let configs = (try? JSONSerialization.jsonObject(with: data, options: [])) as? [String: Any]
        else {
            print("[package] Error: Configuration File is damaged \(url.path)")
            return
        }
        
        setDeprecatedIntegrationConfigs()
        
        print("[package] Info: Configuration File successfully loaded \(url.path)")
        self.configs = configs
    }
    
    private func setDeprecatedIntegrationConfigs() {
        for (key, value) in deprecatedIntegrationConfigs {
            self.configs[key] = value
        }
    }
    
    internal func prepare(withAPIKey apiKey: String?) {
        // API Key
        if let apiKey = apiKey { // Custom API Key
            JedAIApp.apiKey = apiKey
        }
        else if let apiKey = Bundle.main.object(forInfoDictionaryKey: "ANAGOG_API_KEY") as? String { // Support for old integration
            JedAIApp.apiKey = apiKey
        }
        
        // JSON URL
        self.loadFromFile()
        
        // JSON Configurations
        if let value = configs["callCompletionHandlerForRemoteNotifications"] as? Bool {
            AnagogSDKWrapper.callCompletionHandlerForRemoteNotifications = value
        }
        if let value = configs["overwriteUNUserNotificationCenterDelegate"] as? Bool {
            AnagogSDKWrapper.overwriteUNUserNotificationCenterDelegate = value
        }
        if let jedaiAutoEnabled = configs["jedaiAutoEnabled"] as? Bool {
            AnagogStateManager.shared.jedaiAutoEnabled = jedaiAutoEnabled
        }
        if let developmentMode = configs["enableInternalLog"] as? Bool {
            JedAIApp.developmentMode = developmentMode
        }
    }
    
    open override func handle(_ defaultOptions : JedAIAppOptions) -> JedAIAppOptions {
        var options = defaultOptions
        for (key, value) in configs {
            switch key {
            case "enableCrashReports" where value is Bool:
                options.enableCrashReports = value as! Bool

            case "enableRemoteConfiguration" where value is Bool:
                options.enableRemoteConfiguration = value as! Bool
                
            case "locationAware" where value is Bool:
                options.locationAware = value as! Bool
                
            case "enablePOIResolution" where value is Bool:
                options.enablePOIResolution = value as! Bool
                
            case "enableGeofenceDetection" where value is Bool:
                options.enableGeofenceDetection = value as! Bool
                
            case "enableVisitsDetection" where value is Bool:
                options.enableVisitsDetection = value as! Bool
                
            case "enableCyclingDetection" where value is Bool:
                options.enableCyclingDetection = value as! Bool
                
            case "enableSmartVehicleTypeDetection" where value is Bool:
                options.enableSmartVehicleTypeDetection = value as! Bool
                
            case "defaultVehicleType" where value is String:
                if let vehicleType = JedAIKit.JedAIModel.VehicleType.init(rawValue: value as! String) {
                    options.defaultVehicleType = vehicleType
                }
                
            case "enableJEMA" where value is Bool:
                options.enableJEMA = value as! Bool
                
            case "purgeOldData" where value is Bool:
                options.purgeOldData = value as! Bool
                
            case "dataLifetimeFullMonths" where value is Int:
                options.dataLifetimeFullMonths = value as! Int
                
            case "enableRemoteNotifications" where value is Bool,
                "remoteNotifications" where value is Bool: // Support of old json files
                options.enableRemoteNotifications = value as! Bool
                
            case "enableAudienceAnalytics" where value is Bool:
                options.enableAudienceAnalytics = value as! Bool
                
            default:
                options.more[key] = value
            }
        }
        
        // Handlers
        
        options.campaignTriggerActionHandler = { action, userInfo in
            os_log_file("[package] campaignTriggerActionHandler executed with action: %@, userInfo: %@", action ?? "", userInfo)
            var userInfo = userInfo
            userInfo["action"] = action
            if let notification = userInfo["notification"] as? [String: Any],
               let userObject = notification["user_object"] as? [String: Any],
               let customData = userObject["custom_data"] {
                userInfo["custom_data"] = customData
            }
            EventEmitterManager.shared.sendEvent(.onCampaignTriggered, payload: userInfo)
            AnagogSDKWrapper.shared.campaignHandlers.campaignTriggerActionHandler?(action, userInfo)
        }
        options.campaignNotificationClickActionHandler = { action, actionPerformed, userInfo in
            os_log_file("[package] notificationActionExecutionHandler executed with action: %@, success: %@, userInfo: %@", action ?? "", actionPerformed ? "true" : "false", userInfo)
            var payload = userInfo
            payload["executed"] = actionPerformed
            EventEmitterManager.shared.sendEvent(.onNotificationClick, payload: payload)
            AnagogSDKWrapper.shared.campaignHandlers.campaignNotificationClickActionHandler?(action, actionPerformed, userInfo)
        }
        options.campaignNotificationApprovementHandler = { userInfo -> Bool in
            os_log_file("[package] notificationApprovementHandler executed with event: %@", userInfo)
            if let handler = AnagogSDKWrapper.shared.campaignHandlers.campaignNotificationApprovementHandler {
                return handler(userInfo)
            }
            return true
        }
        options.microsegmentsSnapshotReportHandler = { snapshot in
            os_log_file("[package] microsegmentsSnapshotReportHandler executed with payload: %@", snapshot)
            let payload: [String: [String: Any]] = snapshot.mapValues { value in
                var map = [String: Any]()
                if let string = value.stringValue {
                    map["s"] = string
                }
                if let number = value.numericValue {
                    map["n"] = number
                }
                return map
            }
            EventEmitterManager.shared.sendEvent(.onSnapshotReport, payload: payload)
        }
        
        return options
    }
    
}
