//
//  AnagogSDKWrapper.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Foundation
import JedAIAppKit
import JedAIKit
import os

@objc
public class AnagogSDKWrapper: NSObject {
    
    @objc
    public static let shared = AnagogSDKWrapper()
    
    public static var optionsHandler = AnagogPackageOptionsHandler()

    @objc
    public let campaignHandlers = CampaignHandlers()
    
    @objc
    public static var isStartupAllowed: Bool { AnagogStateManager.shared.isStartupAllowed }
    
    @objc
    public internal(set) static var callCompletionHandlerForRemoteNotifications: Bool = true
    @objc
    public internal(set) static var overwriteUNUserNotificationCenterDelegate: Bool = true
    
    internal var unUserNotificationCenterDelegate: UNUserNotificationCenterDelegate? = nil
    
    @objc
    public static var configurationFileURL: URL? {
        get { optionsHandler.url }
        set { optionsHandler.url = newValue }
    }
    
    @objc
    public static var developmentMode: Bool {
        get { JedAIApp.developmentMode }
        set { JedAIApp.developmentMode = newValue }
    }
    
    internal var sdkIsInitialized: Bool = false
    
    
    // MARK: - Deprecated variables
    
    @available(*, deprecated)
    @objc
    public var isLaunchedInBackground: Bool {
        return launchOptions.contains(where: {$0.key == .location || $0.key == .remoteNotification})
    }
    @available(*, deprecated)
    @objc
    public var isLaunchedInForeground: Bool {
        !isLaunchedInBackground
    }
    @available(*, deprecated)
    internal var launchOptions: [UIApplication.LaunchOptionsKey: Any] = [:]
    
    
    // MARK: - Initialization
    
    private override init() {
        _ = AnagogStateManager.shared
        _ = JedAIApp.shared
        
        super.init()
    }
    
    
    // MARK: - New startup
    
    @objc
    public func startup() {
        self.startup(withAPIKey: nil)
    }
    
    @objc
    public func startup(withAPIKey apiKey: String? = nil) {
        Self.optionsHandler.prepare(withAPIKey: apiKey) // Set API Key and loading JSON file
        self.handleUNUserNotificationCenter(force: false) // Handle UNUserNotificationCenter calls
        
        // Startup
        JedAIApp.optionsHandler = Self.optionsHandler
        self.sdkIsInitialized = true
        AnagogStateManager.shared.startup()
    }
    
    
    // MARK: - Configurations
    
    @objc
    public func isLaunchedInBackground(launchOptions: [UIApplication.LaunchOptionsKey: Any]) -> Bool {
        return launchOptions.contains(where: {$0.key == .location || $0.key == .remoteNotification})
    }
    
    @objc
    public func isLaunchedInForeground(launchOptions: [UIApplication.LaunchOptionsKey: Any]) -> Bool {
        return !isLaunchedInBackground(launchOptions: launchOptions)
    }
    
    @objc
    public func handleUNUserNotificationCenter(force: Bool = true) {
        guard force || Self.overwriteUNUserNotificationCenterDelegate else {
            return
        }
        
        if let oldDelegate = UNUserNotificationCenter.current().delegate, oldDelegate !== self {
            unUserNotificationCenterDelegate = oldDelegate
        }
        UNUserNotificationCenter.current().delegate = self
    }
    
    
    // MARK: - AppDelegate methods
    
    @objc
    public func refresh() {
        JedAIApp.shared.refresh()
    }
    
    @objc
    public func didReceiveNotification(_ notification: UNNotification) {
        guard Self.overwriteUNUserNotificationCenterDelegate == false else {
            assertionFailure("Do not call `didReceiveNotification` manually, when `overwriteUNUserNotificationCenterDelegate` option is set to `true`")
            return
        }
        JedAIApp.shared.didReceiveNotification(notification)
    }
    
    @objc
    @discardableResult
    public func didReceiveRemoteNotification(_ userInfo: [AnyHashable: Any]) -> Bool {
        return JedAIApp.shared.didReceiveRemoteNotification(userInfo)
    }
    
    @objc
    public func updateDeviceToken(_ deviceToken: Data) {
        JedAIApp.shared.updateDeviceToken(deviceToken)
    }
    
}
