//
//  AnagogStateManager.swift
//
//  Copyright © 2022 Anagog Ltd. All rights reserved.
//

import Foundation
import JedAIAppKit
import JedAIKit

class AnagogStateManager {
    static let shared = AnagogStateManager()
    
    private let userDefaults = UserDefaults.standard
    private var jedaiStateListener = JedAIStateEvent.Listener()
    
    private let isStartupAllowedKey = "com.anagog.plugin.is-startup-allowed"
    private(set) var isStartupAllowed: Bool {
        get {
            userDefaults.object(forKey: isStartupAllowedKey) as? Bool ?? jedaiAutoEnabled
        }
        set {
            userDefaults.set(newValue, forKey: isStartupAllowedKey)
        }
    }
    
    var jedaiAutoEnabled: Bool = true
    
    private var startupCallback: (() -> ())? = nil
    private var shutdownCallback: (() -> ())? = nil
    
    private init() {
        jedaiStateListener.listen { event in
            if event.state == JedAIState.active {
                EventEmitterManager.shared.sendEvent(.onJedAIStatusChange, payload: ["status": true])
                self.startupCallback?()
                self.startupCallback = nil
                os_log_file("[package] SDK started", type: .info)
            }
            else if event.state == JedAIState.passive {
                EventEmitterManager.shared.sendEvent(.onJedAIStatusChange, payload: ["status": false])
                self.shutdownCallback?()
                self.shutdownCallback = nil
                os_log_file("[package] SDK stopped", type: .info)
            }
        }
    }
    
    func enableJedAI(_ callback: @escaping () -> ()) {
        os_log_file("[package] Enable JedAI", type: .info)
        isStartupAllowed = true
        guard JedAI.shared.state == JedAIState.passive else {
            callback()
            return
        }
        startupCallback = callback
        startup()
    }
    
    func disableJedAI(cleanup: Bool, callback: @escaping () -> ()) {
        os_log_file("[package] Disable JedAI", type: .info)
        isStartupAllowed = false
        guard JedAI.shared.state == JedAIState.active else {
            callback()
            return
        }
        shutdownCallback = callback
        shutdown(fullstop: true, cleanup: cleanup)
    }
    
    func startup() {
        print("[package] Startup")
        guard isStartupAllowed else {
            debugPrint("[package] JedAI startup aborted by configurations")
            return
        }
        guard AnagogSDKWrapper.shared.sdkIsInitialized else {
            assertionFailure("AnagogSDK is not initialized yet")
            return
        }
        
        JedAIApp.shared.startup()
    }
    
    func shutdown(fullstop: Bool, cleanup: Bool) {
        os_log_file("[package] Shutdown with fullstop: %@ and cleanup: %@", type: .info, fullstop ? "true" : "false", cleanup ? "true" : "false")
        guard AnagogSDKWrapper.shared.sdkIsInitialized else {
            assertionFailure("AnagogSDK is not initialized yet")
            return
        }
        
        JedAIApp.shared.shutdown(fullstop: fullstop, cleanup: cleanup)
    }
    
}
