import 'package:anagog_flutter/logger/logger.dart';
import 'package:anagog_flutter/onboarding/onboarding.dart';
import 'package:anagog_flutter/page_tracker/page_tracker.dart';
import 'package:anagog_flutter/platform_interface/anagog_flutter_platform_interface.dart';
import 'package:anagog_flutter/platform_interface/model/campaign.dart';
import 'package:anagog_flutter/platform_interface/model/constants.dart';
import 'package:anagog_flutter/platform_interface/model/event_type.dart';
import 'package:anagog_flutter/platform_interface/model/micromoment.dart';
import 'package:flutter/services.dart';

class AnagogFlutterMethodChannel
    implements
        AnagogFlutterPlatform,
        AnagogFlutterOnboarding,
        AnagogFlutterPageTracker,
        AnagogFlutterLogger {
  final _methodChannel =
      const MethodChannel('com.anagog.flutter/method_channel');
  final _eventChannel =
      const EventChannel('com.anagog.flutter/method_channel/events');

  @override
  Future simulateCampaign(String? campaignIdentifier) async =>
      _methodChannel.invokeMethod('simulateCampaign',
          <String, dynamic>{CAMPAIGN_IDENTIFIER: campaignIdentifier});

  @override
  Future syncCampaigns() async => _methodChannel.invokeMethod('syncCampaigns');

  @override
  Future<Map<dynamic, dynamic>?> getUserDefinedStat() async =>
      _methodChannel.invokeMapMethod("getUserDefinedStat");

  @override
  Future<dynamic> setUserDefinedString(String name, String value) async {
    Map<String, dynamic> params = Map();
    params["key"] = name;
    params["value"] = value;
    _methodChannel.invokeMethod("setUserDefinedString", params);
  }

  @override
  Future<String?> getUserDefinedString(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedString", name);

  @override
  Future<dynamic> setUserDefinedInteger(String name, int value) async {
    Map<String, dynamic> params = Map();
    params["key"] = name;
    params["value"] = value;
    _methodChannel.invokeMethod("setUserDefinedInteger", params);
  }

  @override
  Future<int?> getUserDefinedInteger(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedInteger", name);

  @override
  Future<dynamic> setUserDefinedDecimal(String name, double value) async {
    Map<String, dynamic> params = Map();
    params["key"] = name;
    params["value"] = value;
    _methodChannel.invokeMethod("setUserDefinedDecimal", params);
  }

  @override
  Future<double?> getUserDefinedDecimal(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedDecimal", name);

  @override
  Future fireMicromoments(Micromoment micromoment) async =>
      _methodChannel.invokeMethod("fireMicromoment", micromoment.toJson());

  @override
  Future share() async => _methodChannel.invokeMethod("share");

  @override
  Stream obtainEventStream() => _eventChannel.receiveBroadcastStream();

  @override
  Future getJedAIVersion() async => _methodChannel.invokeMethod("getVersion");

  @override
  Future requestMicrosegmentsSnapshotReport() async =>
      _methodChannel.invokeMethod("requestMicrosegmentsSnapshotReport");

  @override
  Future forceScheduleReports() async =>
      _methodChannel.invokeMethod("forceScheduleReports");

  @override
  Future setUserInteractionEvent(
      String campaignIdentifier, AnagogActivityType activityType) async {
    Map<String, String> params = {};
    params[CAMPAIGN_IDENTIFIER] = campaignIdentifier;
    params[ACTIVITY_TYPE] = activityType.name;
    return _methodChannel.invokeMethod("setUserInteractionEvent", params);
  }

  @override
  Future<List<Campaign>?> getCampaigns() async {
    var result = await _methodChannel.invokeListMethod("getCampaigns");
    try {
      var campaigns = result
          ?.map((e) => Campaign.fromJson(Map<String, dynamic>.from(e)))
          .toList();
      return campaigns;
    } on Exception catch (e) {
      print(
          "AnagogFlutterMethodChannel: getCampaigns - Something went wrong: $e");
      return [];
    }
  }

  @override
  Future getPluginVersion() async =>
      _methodChannel.invokeMethod("getPluginVersion");

  @override
  Future<JedAIStatus> getJedAIStatus() async {
    var result = await _methodChannel.invokeMethod("getJedAIStatus");
    return result ? JedAIStatus.active : JedAIStatus.passive;
  }

  @override
  Future enableJedAI() async => _methodChannel.invokeMethod("enableJedAI");

  @override
  Future disableJedAI() async => _methodChannel.invokeMethod("disableJedAI");

  @override
  Future disableAndPurgeJedAI() async =>
      _methodChannel.invokeMethod("disableAndPurgeJedAI");

  @override
  Future openDebuggingScreen() async =>
      _methodChannel.invokeMethod("openDebuggingScreen");

  /*
  *  Onboarding
  * */
  @override
  Future start() async => _methodChannel.invokeMethod("onboardingStart");

  @override
  Future markAsCompleted() async =>
      _methodChannel.invokeMethod("onboardingMarkAsCompleted");

  @override
  Future complete() async => _methodChannel.invokeMethod("onboardingComplete");

  @override
  Future reset() async => _methodChannel.invokeMethod("onboardingReset");

  /*
   *
   * Page Visit
   */

  @override
  Future enterPage(String pageName) async {
    Map<String, String> params = {};
    params["pageName"] = pageName;
    return _methodChannel.invokeMethod("enterPage", params);
  }

  @override
  Future exitPage() async => _methodChannel.invokeMethod("exitPage");

  /*
   * Logger
   */
  @override
  Future info({required String message}) async {
    Map<String, String> params = {};
    params["message"] = message;
    _methodChannel.invokeMethod("logInfo", params);
  }

  @override
  Future error({required String message}) async {
    Map<String, String> params = {};
    params["message"] = message;
    _methodChannel.invokeMethod("logError", params);
  }
}
